
/*
 * XKOBO, a video-oriented game
 * Copyright (C) 1995  Akira Higuchi
 *     a-higuti@math.hokudai.ac.jp
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 */

#ifndef _XLCMAP
#define _XLCMAP
  
#include "xlbacking.h"


/*
 *
 *
 *      ---  win_backing ---- win_cmap
 *
 *
 */

class win_cmap : public win_backing{

    /************************ color map **************************/
    
  protected:
    Colormap cmap;
    
  public:
    win_cmap();
    ~win_cmap();
    void make(win *back,int wx=0,int wy=0,int sizex=200,int sizey=200);
    void cmset(XColor *col, int n=256);
};

#endif // _XLCMAP

